<?php

/**
 * Manage BuddyPress groups.
 */
class BPCLI_Group extends BPCLI_Component {
	protected $obj_id_key = 'group_id';
	protected $obj_type = 'group';

	/**
	 * Create a group.
	 *
	 * ## OPTIONS
	 *
	 * --name=<name>
	 * : Name of the group.
	 *
	 * [--slug=<slug>]
	 * : URL-safe slug for the group. If not provided, one will be generated automatically.
	 *
	 * [--description=<description>]
	 * : Group description. Default: 'Description for group "[name]"'
	 *
	 * [--creator_id=<creator_id>]
	 * : ID of the group creator. Default: 1.
	 *
	 * [--slug=<slug>]
	 * : URL-safe slug for the group.
	 *
	 * [--status=<status>]
	 * : Group status (public, private, hidden). Default: public.
	 *
	 * [--enable-forum=<enable-forum>]
	 * : Whether to enable legacy bbPress forums. Default: 0.
	 *
	 * [--date-created=<date-created>]
	 * : MySQL-formatted date. Default: current date.
	 *
	 * ## EXAMPLES
	 *
	 *        wp bp group create --name="Totally Cool Group"
	 *        wp bp group create --name="Sports" --description="People who love sports" --creator_id=54 --status=private
	 *
	 * @synopsis --name=<name> [--slug=<slug>] [--description=<description>] [--creator_id=<creator_id>] [--status=<status>] [--enable-forum=<enable-forum>] [--date-created=<date-created>]
	 *
	 * @since 1.0
	 */
	public function create( $args, $assoc_args ) {
		$r = wp_parse_args( $assoc_args, array(
			'name'         => '',
			'slug'         => '',
			'description'  => '',
			'creator_id'   => 1,
			'status'       => 'public',
			'enable_forum' => 0,
			'date_created' => bp_core_current_time(),
		) );

		if ( ! $r['name'] ) {
			WP_CLI::error( 'You must provide a --name parameter when creating a group.' );
		}

		// Auto-generate some stuff.
		if ( ! $r['slug'] ) {
			$r['slug'] = groups_check_slug( sanitize_title( $r['name'] ) );
		}

		if ( ! $r['description'] ) {
			$r['description'] = sprintf( 'Description for group "%s"', $r['name'] );
		}

		$id = groups_create_group( $r );
		if ( $id ) {
			groups_update_groupmeta( $id, 'total_member_count', 1 );
			$group = groups_get_group( array(
				'group_id' => $id,
			) );
			WP_CLI::success( "Group $id created: " . bp_get_group_permalink( $group ) );
		} else {
			WP_CLI::error( 'Could not create group.' );
		}
	}

	/**
	 * Generate random groups.
	 *
	 * ## OPTIONS
	 *
	 * [--count=<number>]
	 * : How many groups to generate. Default: 100
	 *
	 * ## EXAMPLES
	 *
	 *  wp bp group generate --count=50
	 *
	 * @synopsis [--count=<number>]
	 *
	 * @since 1.3.0
	 */
	public function generate( $args, $assoc_args ) {
		$r = wp_parse_args( $assoc_args, array(
			'count' => 100,
		) );

		$notify = \WP_CLI\Utils\make_progress_bar( 'Generating groups', $r['count'] );

		for ( $i = 0; $i < $r['count']; $i++ ) {
			$this->create( array(), array(
				'name' => 'Test Group - ' . $i,
			) );

			$notify->tick();
		}

		$notify->finish();
	}

	/**
	 * Delete a group.
	 *
	 * ## OPTIONS
	 *
	 * <group-id>
	 * : Identifier for the group. Can be a numeric ID or the group slug.
	 *
	 * ## EXAMPLES
	 *
	 *  wp bp group delete 500
	 *
	 * @synopsis <group-id>
	 *
	 * @since 1.3.0
	 */
	public function delete( $args, $assoc_args ) {
		$group_id = isset( $args[0] ) ? $args[0] : '';

		if ( ! is_numeric( $group_id ) ) {
			$group_id = groups_get_id( $group_id );
		}

		// Check that group exists.
		$group_obj = groups_get_group( array(
			'group_id' => $group_id,
		) );
		if ( empty( $group_obj->id ) ) {
			WP_CLI::error( 'No group found by that slug or ID.' );
		}

		// Delete group. True if deleted.
		if ( groups_delete_group( $group_id ) ) {
			WP_CLI::success( 'Group deleted.' );
		} else {
			WP_CLI::error( 'Could not delete the group.' );
		}
	}

	/**
	 * Update a group.
	 *
	 * ## OPTIONS
	 *
	 * <group-id>
	 * : Identifier for the group. Can be a numeric ID or the group slug.
	 *
	 * --<field>=<value>
	 * : One or more fields to update. See groups_create_group()
	 *
	 * ## EXAMPLES
	 *
	 *     wp bp group update 35 --description="What a cool group!" --name="Group of Cool People"
	 */
	public function update( $args, $assoc_args ) {
		$clean_group_ids = array();

		foreach ( $args as $group_id ) {
			// Convert --group_id to group ID
			// @todo this'll be screwed up if the group has a numeric slug.
			if ( ! is_numeric( $group_id ) ) {
				$group_id = groups_get_id( $group_id );
			}

			// Check that group exists.
			$group_obj = groups_get_group( array(
				'group_id' => $group_id,
			) );
			if ( empty( $group_obj->id ) ) {
				WP_CLI::error( 'No group found by that slug or ID.' );
			}

			$clean_group_ids[] = $group_id;
		}

		parent::_update( $clean_group_ids, $assoc_args, function( $params ) {
			return groups_create_group( $params );
		} );
	}

	/**
	 * Get a list of groups.
	 *
	 * ## OPTIONS
	 *
	 * --<field>=<value>
	 * : One or more parameters to pass. See groups_get_groups()
	 *
	 * [--format=<format>]
	 * : Render output in a particular format.
	 *
	 * ## EXAMPLES
	 *
	 *   wp bp group list --format=ids
	 *
	 * @since 1.3.0
	 */
	public function list_( $args, $assoc_args ) {
		$r = wp_parse_args( $args, array(
			'type' => 'active',
		) );

		$args      = array_merge( $r, $assoc_args );
		$formatter = $this->get_formatter( $args );

		if ( 'ids' === $formatter->format ) {
			$args['fields']      = 'ids';
			$args['show_hidden'] = true;
			$args['per_page']    = null; // Return all results.

			$groups  = groups_get_groups( $args );
			$groups  = $groups['groups'];
			echo implode( ' ', $groups ); // XSS ok.
		} else {
			$groups  = groups_get_groups( $args );
			$groups  = $groups['groups'];

			$formatter->display_items( $groups );
		}
	}

	/**
	 * Add a member to a group.
	 *
	 * ## OPTIONS
	 *
	 * --group-id=<group>
	 * : Identifier for the group. Accepts either a slug or a numeric ID.
	 *
	 * --user-id=<user>
	 * : Identifier for the user. Accepts either a user_login or a numeric ID.
	 *
	 * [--role=<role>]
	 * : Group role for the new member (member, mod, admin). Default: member.
	 *
	 * ## EXAMPLES
	 *
	 *        wp bp group add_member --group-id=3 --user-id=10
	 *        wp bp group add_member --group-id=foo --user-id=admin role=mod
	 *
	 * @synopsis --group-id=<group> --user-id=<user> [--role=<role>]
	 *
	 * @since 1.0
	 */
	public function add_member( $args, $assoc_args ) {
		$r = wp_parse_args( $assoc_args, array(
			'group-id' => null,
			'user-id'  => null,
			'role'     => 'member',
		) );

		// Convert --group_id to group ID
		// @todo this'll be screwed up if the group has a numeric slug.
		if ( ! is_numeric( $r['group-id'] ) ) {
			$group_id = groups_get_id( $r['group-id'] );
		} else {
			$group_id = $r['group-id'];
		}

		// Check that group exists.
		$group_obj = groups_get_group( array(
			'group_id' => $group_id,
		) );
		if ( empty( $group_obj->id ) ) {
			WP_CLI::error( 'No group found by that slug or id.' );
		}

		$user = $this->get_user_id_from_identifier( $r['user-id'] );

		if ( ! $user ) {
			WP_CLI::error( 'No user found by that username or id' );
		}

		// Sanitize role.
		if ( ! in_array( $r['role'], array( 'member', 'mod', 'admin' ), true ) ) {
			$r['role'] = 'member';
		}

		$joined = groups_join_group( $group_id, $user->ID );

		if ( $joined ) {
			if ( 'member' !== $r['role'] ) {
				$the_member = new BP_Groups_Member( $user->ID, $group_id );
				$the_member->promote( $r['role'] );
			}

			$success = sprintf(
				'Added user #%d (%s) to group #%d (%s) as %s',
				$user->ID,
				$user->user_login,
				$group_id,
				$group_obj->name,
				$r['role']
			);
			WP_CLI::success( $success );
		} else {
			WP_CLI::error( 'Could not add user to group.' );
		}
	}

	/**
	 * Get a list of members for a group.
	 *
	 * ## OPTIONS
	 *
	 * <group-id>
	 * : Identifier for the group. Accepts either a slug or a numeric ID.
	 *
	 * ## EXAMPLES
	 *
	 *        wp bp group get_members 3
	 *
	 * @synopsis <group-id>
	 *
	 * @since 1.3.0
	 */
	public function get_members( $args, $assoc_args ) {
		$group_id = isset( $args[0] ) ? $args[0] : '';

		// Convert --group_id to group ID
		// @todo this'll be screwed up if the group has a numeric slug.
		if ( ! is_numeric( $group_id ) ) {
			$group_id = groups_get_id( $group_id );
		}

		// Check that group exists.
		$group_obj = groups_get_group( array(
			'group_id' => $group_id,
		) );
		if ( empty( $group_obj->id ) ) {
			WP_CLI::error( 'No group found by that slug or id.' );
		}

		// Get our members.
		$members = groups_get_group_members( array(
			'group_id' => $group_id,
		) );

		if ( $members['count'] ) {
			$found = sprintf(
				'Found %d members in group #%d',
				$members['count'],
				$group_id
			);
			WP_CLI::success( $found );

			$member_list = implode( ', ', wp_list_pluck( $members['members'], 'user_login' ) );

			$users = sprintf(
				'Current members for group #%d: %s',
				$group_id,
				$member_list
			);

			WP_CLI::success( $users );
		} else {
			WP_CLI::error( 'Could not find any users in the group.' );
		}
	}
}

WP_CLI::add_command( 'bp group', 'BPCLI_Group', array(
	'before_invoke' => function() {
		if ( ! bp_is_active( 'groups' ) ) {
			WP_CLI::error( 'The Groups component is not active.' );
		}
	},
) );

